@@ -288,6 +288,13 @@ def parse_article_xml(xml_file):
                 % utils.match_control_character_entities(xml_string)
             )
             xml_string = utils.replace_control_character_entities(xml_string)
+        # also replace unescaped control characters
+        if utils.match_control_characters(xml_string):
+            LOGGER.info(
+                "Replacing control characters in the XML string (ASCII codes): %s"
+                % [ord(char) for char in utils.match_control_characters(xml_string)]
+            )
+            xml_string = utils.replace_control_characters(xml_string)
         try:
             return ElementTree.fromstring(xml_string)
         except ElementTree.ParseError:
@@ -453,3 +460,15 @@ def pdf_page_count(file_path):
             )
             raise
     return None
+
+
+def preprint_url(root):
+    "parse XML to find a preprint URL"
+    url = None
+    for fn_group_tag in root.findall(
+        './front/article-meta/fn-group[@content-type="article-history"]'
+    ):
+        ext_link_tag = fn_group_tag.find('./ext-link[@ext-link-type="url"]')
+        if ext_link_tag is not None:
+            url = ext_link_tag.get("{http://www.w3.org/1999/xlink}href")
+    return url
